import { useEffect, useRef, useState } from 'react';
import styles from './Emulator.module.scss';
import { GAME_SYSTEM_CORES, GameSystem, GameSystemCore } from '@/lib/game';

type SendEmulatorMessageInit = {
  message:'init';
  core:GameSystemCore;
  system:GameSystem;
  gameId:string;
  gameName:string;
}

type SendEmulatorMessageVolume = {
  message:'volume';
  volume:number;
};

type SendEmulatorMessage = (
  SendEmulatorMessageInit |
  SendEmulatorMessageVolume
);

type ReceiveEmulatorMessage = (
  { message: 'start' } |
  { message: 'ready' } |
  { message: 'save' } |
  { message: 'load' }
);

export type EmulatorProps = {
  system:GameSystem;
  gameId:string;
  gameName:string;
};

export const Emulator:React.FC<EmulatorProps> = props => {
  const iframeRef = useRef<HTMLIFrameElement>(null);
  const [ hasInit, setHasInit ] = useState(false);

  const send = (msg:SendEmulatorMessage) => {
    iframeRef.current?.contentWindow?.postMessage(msg, '*');
  };

  useEffect(() => {
    if(hasInit) return;
    setHasInit(true);

    send({
      message: 'init',
      core: GAME_SYSTEM_CORES[props.system],
      system: props.system,
      gameId: props.gameId,
      gameName: props.gameName
    });

    window.onmessage = e => {
      if(!e.data) {
        console.error('Invalid message received:', e.data);
        return;
      }
      
      const msg = e.data as ReceiveEmulatorMessage;
      switch(msg.message) {
        case 'start':
        case 'ready':
        case 'load':
        case 'save':
          break;
        default:
          console.error('Invalid message received:', msg);
          break
      }
    };
  }, [ iframeRef ]);

  return (
    <div className={styles.emulator}>
      <div className={[
        styles.emulator__box,
        styles[`emulator__box--${props.system}`]
      ].join(' ')} />

      <iframe
        src="/emulator.html"
        className={styles.emulator__iframe}
        ref={iframeRef}
      />
    </div>
  );
}