import { NextApiHandler } from "next";
import * as fs from 'fs';
import * as path from 'path';

const PATH_SAVES = path.resolve('.', 'data', 'saves');

const handler:NextApiHandler = async (req, res) => {
  if (req.method !== 'GET') {
    res.setHeader('Allow', ['GET']);
    res.status(405).end(`Method Not Allowed`);
    return;
  }

  // ID query param
  if(!req.query.id || typeof req.query.id !== 'string') {
    res.status(404).end(`Not Found`);
    return;
  }

  const { id } = req.query;
  if(!id) {
    res.status(404).end(`Not Found`);
    return;
  }

  // Does rom exist?
  const pathSave = path.resolve(PATH_SAVES, `${id}.zip`);
  if(!fs.existsSync(pathSave)) {
    res.status(404).end(`Not Found`);
    return;
  }

  // Read save
  try {
    const romStream = fs.createReadStream(pathSave);
    res.setHeader('Content-Type', 'application/octet-stream');
    res.setHeader('Content-Disposition', `attachment; filename="${id}.zip"`);
    romStream.pipe(res);
  } catch (error) {
    res.status(500).end(`Internal Server Error`);
  }
};

export default handler;