import { apiClientGet } from '@/lib/apiClient';
import { includeFragment, extendFragment } from '@/lib/fragment';
import { GameHeavy, GameLightFragment } from '@/lib/game';
import { Error500, Error500Props } from '@/pages/500';
import { gql } from 'apollo-server-micro';
import { GetServerSideProps } from 'next';
import Link from 'next/link';

type PageParams = {
  id:string;
}

type PageProps = {
  game:GameHeavy;
} | Error500Props;

export const getServerSideProps:GetServerSideProps<PageProps, PageParams> = async ({ params }) => {
  try {
    if(!params || !params.id) return { notFound: true };

    const client = await apiClientGet();
    const res = await client.query<{ game:GameHeavy }>({
      variables: { id: params.id },
      query: gql`
        ${includeFragment(GameLightFragment)}

        query getGame($id: ID!) {
          game(id: $id) {
            ${extendFragment(GameLightFragment)}
          }
        }
      `,
    });
    if(!res.data.game) return { notFound: true };

    return { props: { ...res.data } };
  } catch(e) {
    console.error('Error', e);
    return { props: { code: 500 } };
  }
};

export const Page:React.FC<PageProps> = props => {
  if('code' in props) {
    if(props.code === 500) return <Error500 {...props} />;
    return null;
  }

  const { game } = props;

  return (
    <div>
      <h1>{ game.name }</h1>

      <Link href={`/games/${game.id}/play`}>
        Play Game
      </Link>
    </div>
  );
};

export default Page;